@@ -3,8 +3,11 @@ import os
 from typing import Dict, List, Literal, Optional, OrderedDict, Union
 
 import hiyapyco
+from prompt_toolkit.layout import D, Dimension
 from prompt_toolkit.output import ColorDepth
 
+from app.util.interpolation import Interpolation, parse_interpolations
+
 
 class MisconfigurationError(Exception):
     pass
@@ -26,7 +29,7 @@ class ProcessConfig:
     cmd: Optional[List[str]] = None
     cwd: str = os.getcwd()
     stop: str = "SIGKILL"
-    env: Dict[str, Optional[str]] = None
+    env: Optional[Dict[str, Optional[str]]] = None
     add_path: Optional[Union[str, List[str]]] = None
     description: Optional[str] = None
     docs: Optional[str] = None
@@ -40,6 +43,12 @@ class ProcessConfig:
         if not self.cmd and not self.shell:
             raise MisconfigurationError('shell or cmd is required for every proc definition')
 
+    @property
+    def interpolations(self) -> List[Interpolation]:
+        if self.cmd:
+            return parse_interpolations(*self.cmd)
+        return parse_interpolations(self.shell)
+
 
 @dataclass
 class LayoutConfig:
@@ -48,6 +57,7 @@ class LayoutConfig:
     processes_list_width: int = 31
     sort_process_list_alpha: bool = True
     category_search_prefix: str = 'cat:'
+    field_replacement_prompt: str = '__FIELD_NAME__ ⮕  '
 
 
 @dataclass
@@ -76,12 +86,24 @@ class StyleConfig:
             return ColorDepth.DEFAULT
         return ColorDepth.TRUE_COLOR
 
+    @property
+    def width_100(self) -> Dimension:
+        return D(preferred=100 * 100)
+
+    @property
+    def height_100(self) -> Dimension:
+        return D(preferred=100 * 100)
+
 
 @dataclass
 class KeybindingConfig:
     quit: List[str] = field(default_factory=lambda: ['q'])
     filter: List[str] = field(default_factory=lambda: ['/'])
     submit_filter: List[str] = field(default_factory=lambda: ['enter'])
+    next_input: List[str] = field(default_factory=lambda: ['tab', 'down'])
+    previous_input: List[str] = field(default_factory=lambda: ['s-tab', 'up'])
+    submit_dialog: List[str] = field(default_factory=lambda: ['enter'])
+    cancel_dialog: List[str] = field(default_factory=lambda: ['escape'])
     start: List[str] = field(default_factory=lambda: ['s'])
     stop: List[str] = field(default_factory=lambda: ['x'])
     up: List[str] = field(default_factory=lambda: ['up', 'k'])
@@ -101,7 +123,7 @@ class KeybindingConfig:
 
 @dataclass
 class ProcMuxConfig:
-    procs: Optional[Dict[str, ProcessConfig]]
+    procs: Dict[str, ProcessConfig] = field(default_factory=lambda: {})
     style: StyleConfig = StyleConfig()
     keybinding: KeybindingConfig = KeybindingConfig()
     shell_cmd: List[str] = field(default_factory=lambda: [os.environ['SHELL'], '-c'])
